# coding=utf-8

from hiword import extract_keywords


def test_extract_keywords():
    s = "此外，公司拟对全资子公司吉林欧亚置业有限公司增资4.3亿元，增资后，吉林欧亚置业注册资本由7000万元增加到5亿元。吉林欧亚置业主要经营范围为房地产开发及百货零售等业务。目前在建吉林欧亚城市商业综合体项目。2013年，实现营业收入0万元，实现净利润-139.13万元。"
    for x, w in extract_keywords(s, with_weight=True):
        print(x, w)



# coding=utf-8

import math
from collections import defaultdict

from jieba import Tokenizer

from hiword.dataloader import DictLoader, IDFLoader

__all__ = ['KeywordsExtractor', 'extract_keywords']


class KeywordsExtractor:
    NUMERIC = '0123456789〇一二三四五六七八九十百千万亿元角分'

    def __init__(self, topk=20, min_support=2, dict_file=None, idf_file=None):
        self.tokenizer = Tokenizer()
        self.topk = topk
        self.min_support = min_support
        self.dict = DictLoader(dict_file)
        self.idf = IDFLoader(idf_file)

    def extract_keywords(self, *docs, topn=20, with_weight=False):
        freq = defaultdict(lambda: 0)
        doc_freq = defaultdict(lambda: 0)
        for d in docs:
            if isinstance(d, str):
                words = []
                for i in self.tokenizer.cut(d):
                    w = i.strip()
                    if len(w) == 0:
                        continue
                    words.append(w)
            else:
                words = d
            keywords = self._extract_keywords_from_single_doc(words)
            seg = self._detect_long_keywords(words, keywords)
            f = defaultdict(lambda: 0)
            for w in seg:
                if self._filter_word(w):
                    continue
                f[w] = f[w] + 1
            for k, v in f.items():
                doc_freq[k] = doc_freq[k] + 1
                freq[k] = freq[k] + v
        res = []
        for k in freq:
            s = freq[k] / (freq[k] + self.dict.word_freq(k))
            s *= math.log2(freq[k] + 1) * math.log2(doc_freq[k] + 1)
            s *= math.log2(1 + len(docs) / doc_freq[k])
            res.append((k, s))
        res.sort(key=lambda x: x[1], reverse=True)
        res = res[:topn]
        if with_weight:
            return res
        return [i[0] for i in res]

    def _detect_long_keywords(self, words, keywords):
        if not isinstance(keywords, set):
            keywords = set(keywords)
        appears = defaultdict(lambda: 0)
        ran = []
        w = []

        def commit():
            i = 0
            for i in range(len(w), 0, -1):
                if self.dict.word_pos(w[i - 1]) not in ('v',):
                    break
            if i > 0:
                word = ''.join(w[:i])
                appears[word] += 1
                ran.append((cur - i, cur, word))

        for cur in range(0, len(words)):
            x = words[cur]
            if x in keywords:
                w.append(x)
            elif w:
                commit()
                w = []
        if w:
            commit()

        seg = []
        last = 0
        for begin, end, word in ran:
            if begin > last:
                for i in range(last, begin):
                    seg.append(words[i])
            if appears[word] < self.min_support:
                for i in range(begin, end):
                    seg.append(words[i])
            else:
                seg.append(word)
        for i in range(last, len(words)):
            seg.append(words[i])
        return seg

    def _extract_keywords_from_single_doc(self, words):
        freq = {}
        for w in words:
            if len(w) < 2:
                continue
            freq[w] = freq.get(w, 0.0) + 1.0
        total = sum(freq.values())
        for k in freq:
            freq[k] *= self.idf.word_idf(k) / total
        keywords = sorted(freq, key=freq.__getitem__, reverse=True)
        return keywords[:self.topk]

    def _filter_word(self, w):
        def isnumber(v):
            if v:
                v = v.strip('%')
            try:
                v = float(v)
            except ValueError:
                return False
            return True

        if isnumber(w):
            return True
        if len(w) <= 1:
            return True
        nc = 0
        for i in w:
            if i in self.NUMERIC:
                nc += 1
        if nc * 2 >= len(w):
            return True
        return False


def extract_keywords(*docs, topn=20, with_weight=False, **kwargs):
    extractor = KeywordsExtractor(**kwargs)
    return extractor.extract_keywords(*docs, topn=topn, with_weight=with_weight)



# coding=utf-8

from os.path import join, dirname
from multiprocessing import Lock

default_dict = {}
dict_loader_lock = Lock()

default_idf = {}
idf_loader_lock = Lock()


class DictLoader:
    def __init__(self, dict_file):
        self.dict = self._load_dict(dict_file)

    def word_freq(self, word):
        if word in self.dict:
            return self.dict[word][0]
        return 0

    def word_pos(self, word):
        if word in self.dict:
            return self.dict[word][1]
        return None

    @staticmethod
    def _load_dict(dict_file):
        def read_dict():
            with open(dict_file) as f:
                while True:
                    line = f.readline()
                    if not line:
                        break
                    s = line.split()
                    res[s[0]] = (int(s[1]), s[2])

        if dict_file is None:
            with dict_loader_lock:
                if len(default_dict) > 0:
                    return default_dict
                dict_file = join(dirname(__file__), 'data', 'dict.txt')
                res = default_dict
                read_dict()
        else:
            res = {}
            read_dict()
        return res


class IDFLoader:
    def __init__(self, idf_file):
        self.idf = self._load_idf(idf_file)
        self.median_idf = sorted(self.idf.values())[len(self.idf) // 2]

    def word_idf(self, word):
        return self.idf.get(word, self.median_idf)

    @staticmethod
    def _load_idf(idf_file):
        def read_idf():
            with open(idf_file) as f:
                while True:
                    line = f.readline()
                    if not line:
                        break
                    s = line.split()
                    res[s[0]] = float(s[1])

        if idf_file is None:
            with idf_loader_lock:
                if len(default_idf) > 0:
                    return default_dict
                idf_file = join(dirname(__file__), 'data', 'idf.txt')
                res = default_idf
                read_idf()
        else:
            res = {}
            read_idf()
        return res



# coding=utf-8

import re
import sys
from os.path import join, dirname
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

with open(join(dirname(__file__), 'README.rst'), 'r', encoding='utf-8') as fd:
    long_description = fd.read()


def read_version():
    p = join(dirname(__file__), 'hiword', '__init__.py')
    with open(p, 'r', encoding='utf-8') as f:
        return re.search(r"__version__ = '([^']+)'", f.read()).group(1)


def read_requirements(file):
    with open(join(dirname(__file__), 'requirements', file), 'r', encoding='utf-8') as f:
        return [l.strip() for l in f]


class PyTest(TestCommand):
    def run_tests(self):
        import pytest

        errno = pytest.main(['tests'])
        sys.exit(errno)


tests_require = read_requirements('test.txt')
install_requires = ['jieba>=0.39']


def main():
    if sys.version_info < (3, 5):
        raise RuntimeError("The minimal supported Python version is 3.5")

    setup(
        name="hiword",
        version=read_version(),
        url="https://github.com/jadbin/hiword",
        description="Hi, word!",
        long_description=long_description,
        author="jadbin",
        author_email="jadbin.com@hotmail.com",
        license="Apache 2",
        zip_safe=False,
        packages=find_packages(exclude=("tests",)),
        include_package_data=True,
        python_requires='>=3.5',
        install_requires=install_requires,
        tests_require=tests_require,
        cmdclass={"test": PyTest},
        classifiers=[
            "License :: OSI Approved :: Apache Software License",
            "Intended Audience :: Developers",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Topic :: Software Development :: Libraries :: Python Modules"
        ]
    )


if __name__ == "__main__":
    main()
